package de.ciluvien.mensen

import android.app.PendingIntent
import android.content.Intent
import android.content.IntentFilter
import android.nfc.NfcAdapter
import android.nfc.Tag
import android.nfc.tech.IsoDep
import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.Modifier
import androidx.core.content.IntentCompat
import androidx.navigation.NavHostController
import androidx.navigation.compose.rememberNavController
import de.ciluvien.mensen.data.nfc.INTENT_ACTION_NFC_READ
import de.ciluvien.mensen.ui.Navigation
import de.ciluvien.mensen.ui.Screen
import de.ciluvien.mensen.ui.theme.MensenTheme
import java.util.Timer
import kotlin.concurrent.schedule

class MainActivity : ComponentActivity() {
    private var nfcAdapter : NfcAdapter? = null
    private var navHostController: NavHostController? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        nfcAdapter = NfcAdapter.getDefaultAdapter(this)

        enableEdgeToEdge()
        setContent {
            LaunchedEffect(key1 = navHostController) {
                if(intent.action == NfcAdapter.ACTION_TECH_DISCOVERED) {
                    useNFCIntent(intent)
                }
            }
            MensenTheme {
                navHostController = rememberNavController()
                Surface(
                    modifier = Modifier.fillMaxSize(),
                    color = MaterialTheme.colorScheme.background
                ) {
                    navHostController?.let {
                        Navigation(it)
                    }
                }
            }
        }
    }

    override fun onResume() {
        super.onResume()
        nfcAdapter?.let { adapter ->
            val intent = Intent(this, javaClass).apply {
                flags = Intent.FLAG_ACTIVITY_SINGLE_TOP
                `package` = applicationContext.packageName
            }
            val pendingIntent = PendingIntent.getActivity(
                this,
                0,
                intent,
                PendingIntent.FLAG_MUTABLE
            )
            val nfcIntentFilter = arrayOf(
                IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED)
            )
            val techList = arrayOf(
                arrayOf(
                    IsoDep::class.java.name
                )
            )
            adapter.enableForegroundDispatch(
                this, pendingIntent, nfcIntentFilter, techList
            )
        }
    }

    override fun onPause() {
        super.onPause()
        nfcAdapter?.disableForegroundDispatch(this)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        if (NfcAdapter.ACTION_TECH_DISCOVERED == intent.action) {
            useNFCIntent(intent)
        }
    }

    private fun useNFCIntent(intent: Intent) {
        navHostController?.navigate(Screen.CardScreen.route) {
            launchSingleTop = true
            restoreState = true
        }
        Timer().schedule(250) { // Delay broadcast until navigation has finished
            sendBroadcast(Intent(INTENT_ACTION_NFC_READ).apply {
                Log.i("nfc", "Sending Broadcast")
                putExtra(
                    NfcAdapter.EXTRA_TAG,
                    IntentCompat.getParcelableExtra(intent, NfcAdapter.EXTRA_TAG, Tag::class.java)
                )
            })
        }
    }
}
