package de.ciluvien.mensen.data.remote

import de.ciluvien.mensen.data.local.Meal
import kotlinx.serialization.Serializable
import java.math.BigDecimal
import java.time.LocalDate

@Serializable
data class MealResponse(
    val category: String,
    val id: Int,
    val image: String,
    val name: String,
    val notes: List<String>,
    val prices: Map<String, String>,
    val url: String
) {
    fun getGeneralNotes(): List<String> {
        return notes.filter {
                string -> !string.contains("(") && string.isNotBlank()
        }.map {
                string -> string.trim()
        }
    }

    fun getAdditives(): List<String> {
        return notes.filter {
                string -> string.contains("""\(\d+\)""".toRegex())
        }.map {
                string -> string.trim()
        }
    }

    fun getAllergens(): List<String> {
        return notes.filter {
                string -> string.contains("""\(\w+\)""".toRegex())
        }.filter {
                string -> !getAdditives().contains(string.trim())
        }.map {
                string -> string.trim()
        }
    }

    fun isVegan(): Boolean {
        return notes.stream().anyMatch() {string ->
            string.contains("vegan")
        }
    }

    fun isVegetarian(): Boolean {
        return notes.stream().anyMatch { string ->
            string.contains("vegetarisch")
        } || this.isVegan()
    }

}
