package de.ciluvien.mensen.data.settings

import android.content.Context
import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.Preferences
import androidx.datastore.preferences.core.booleanPreferencesKey
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.preferencesDataStore
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.map

class SettingsService(private val context: Context) {

    private val Context.dataStore: DataStore<Preferences> by preferencesDataStore(name = "settings")
    private object SettingsKeys {
        val isVegetarian = booleanPreferencesKey("vegetarian")
        val isVegan = booleanPreferencesKey("vegan")
    }

    suspend fun enableVegetarian() {
        context.dataStore.edit {
            it[SettingsKeys.isVegetarian] = true
        }
    }

    suspend fun disableVegetarian() {
        context.dataStore.edit {
            it[SettingsKeys.isVegetarian] = false
        }
    }

    suspend fun enableVegan() {
        context.dataStore.edit {
            it[SettingsKeys.isVegan] = true
        }
    }

    suspend fun disableVegan() {
        context.dataStore.edit {
            it[SettingsKeys.isVegan] = false
        }
    }

    val isVeganFlow: Flow<Boolean> = context.dataStore.data.map {
        it[SettingsKeys.isVegan] ?: false
    }

    val isVegetarianFlow: Flow<Boolean> = context.dataStore.data.map {
        it[SettingsKeys.isVegetarian] ?: false
    }
}