package de.ciluvien.mensen.ui

import androidx.compose.runtime.Composable
import androidx.navigation.NavHostController
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.navArgument
import de.ciluvien.mensen.ui.screens.CanteenDetailsScreen
import de.ciluvien.mensen.ui.screens.CanteensScreen
import de.ciluvien.mensen.ui.screens.ImageScreen
import de.ciluvien.mensen.ui.screens.CardScreen
import de.ciluvien.mensen.ui.screens.Libraries
import de.ciluvien.mensen.ui.screens.MapScreen
import de.ciluvien.mensen.ui.screens.MealDetailsScreen
import de.ciluvien.mensen.ui.screens.MenuScreen
import de.ciluvien.mensen.ui.screens.SettingsScreen
import java.time.LocalDate

@Composable
fun Navigation(navController: NavHostController) {
    NavHost(navController = navController, startDestination = Screen.CanteenScreen.route){
        composable(route = Screen.CanteenScreen.route) {
            CanteensScreen(navController)
        }
        composable(
            route = Screen.MenuScreen.route + "?position={position}?date={date}",
            arguments = listOf(
                navArgument("position") {
                    type = NavType.IntType
                    defaultValue = 0
                },
                navArgument("date") {
                    type = NavType.StringType
                    defaultValue = LocalDate.now().toString()
                }
            )
        ) { entry ->
                MenuScreen(
                    navController = navController,
                    initialPosition = entry.arguments?.getInt("position")!!,
                    date = LocalDate.parse(entry.arguments?.getString("date"))
                )
        }
        composable(
            route = Screen.MealDetailsScreen.route + "/{canteenName}/{mealId}",
            arguments = listOf(
                navArgument("canteenName") {
                    type = NavType.StringType
                },
                navArgument("mealId") {
                    type = NavType.IntType
                }
            )
        ){entry ->
            MealDetailsScreen(
                navController = navController,
                canteenName = entry.arguments?.getString("canteenName")!!,
                mealId = entry.arguments?.getInt("mealId")!!
            )

        }
        composable(
            route = Screen.CanteenDetailsScreen.route + "/{canteenId}",
            arguments = listOf(
                navArgument("canteenId") {
                    type = NavType.IntType
                }
            )
        ) {entry ->
            CanteenDetailsScreen(
                navController = navController,
                canteenId = entry.arguments?.getInt("canteenId")!!
            )
        }
        composable(
            route = Screen.MapScreen.route
        ) {
            MapScreen(navController = navController)
        }
        composable(
            route = Screen.SettingsScreen.route
        ) {
            SettingsScreen(navController = navController)
        }
        composable(
            route = Screen.LibrariesScreen.route
        ) {
            Libraries(navController = navController)
        }
        composable(
            route = Screen.ImageScreen.route + "/{imageURL}",
            arguments = listOf(
                navArgument("imageURL") {
                    type = NavType.StringType
                }
            )
        ) { entry ->
            ImageScreen(
                navController = navController,
                imageURL = entry.arguments?.getString("imageURL") ?: ""
            )
        }
        composable(
            route = Screen.CardScreen.route
        ) {
            CardScreen(navController = navController)
        }
    }
}