package de.ciluvien.mensen.ui.components.settings

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Slider
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import de.ciluvien.mensen.R
import de.ciluvien.mensen.ui.viewmodels.SettingsViewModel
import kotlin.math.roundToInt

@Composable
fun Filters(settingsViewModel: SettingsViewModel) {
    val context = LocalContext.current

    Card(
        colors = CardDefaults.cardColors(
            containerColor = MaterialTheme.colorScheme.surfaceContainer,
        ),
        border = BorderStroke(2.dp, MaterialTheme.colorScheme.surfaceContainer),
        modifier = Modifier
            .fillMaxWidth()
            .padding(2.dp)
    ) {
        Column(
            modifier = Modifier
                .fillMaxWidth()
                .padding(10.dp),
        ) {
            Text(
                text = context.resources.getText(R.string.settings_filters).toString(),
                style = MaterialTheme.typography.headlineSmall
            )

            Spacer(modifier = Modifier.size(10.dp))

            Column(
                horizontalAlignment = Alignment.CenterHorizontally,
                modifier = Modifier.fillMaxWidth()
            ) {
                val isVegan = settingsViewModel.isVegan.collectAsState()
                val isVegetarian = settingsViewModel.isVegetarian.collectAsState()
                val isEverything = !isVegan.value && !isVegetarian.value

                val veganColor = if (isVegan.value || isVegetarian.value || isEverything) {
                    MaterialTheme.colorScheme.onSurface
                } else {
                    MaterialTheme.colorScheme.onSurfaceVariant
                }

                val vegetarianColor = if (isVegetarian.value || isEverything) {
                    MaterialTheme.colorScheme.onSurface
                } else {
                    MaterialTheme.colorScheme.onSurfaceVariant
                }

                val everythingColor = if (isEverything) {
                    MaterialTheme.colorScheme.onSurface
                } else {
                    MaterialTheme.colorScheme.onSurfaceVariant
                }

                Row(
                    verticalAlignment = Alignment.Bottom,
                    horizontalArrangement = Arrangement.SpaceBetween,
                    modifier = Modifier.fillMaxWidth()
                ) {
                    Row(
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(
                            text = context.resources.getText(R.string.vegan).toString(),
                            style = MaterialTheme.typography.titleLarge,
                            color = veganColor
                        )
                        Spacer(Modifier.size(5.dp))
                        Icon(
                            painter = painterResource(id = R.drawable.leaf_24),
                            contentDescription = context.resources.getText(R.string.vegan).toString()
                        )
                    }
                    Text(
                        text = context.resources.getText(R.string.settings_filter_everything).toString(),
                        style = MaterialTheme.typography.titleLarge,
                        color = everythingColor
                    )
                }

                var sliderPosition = SettingsViewModel.Filter.EVERYTHING.ordinal
                if(isVegan.value) {
                    sliderPosition = SettingsViewModel.Filter.VEGAN.ordinal
                } else if (isVegetarian.value) {
                    sliderPosition = SettingsViewModel.Filter.VEGETARIAN.ordinal
                }

                Row(
                    Modifier.fillMaxWidth(0.9f)
                ) {
                    Slider(
                        value = sliderPosition.toFloat() / 2,
                        onValueChange = {
                            settingsViewModel.setFilter((it * 2).roundToInt())
                        },
                        steps = 1
                    )
                }

                Row(
                    verticalAlignment = Alignment.Top,
                    horizontalArrangement = Arrangement.Center,
                    modifier = Modifier.fillMaxWidth()
                ) {
                    Row(
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(
                            text = context.resources.getText(R.string.vegetarian).toString(),
                            style = MaterialTheme.typography.titleLarge,
                            color = vegetarianColor
                        )
                        Spacer(Modifier.size(5.dp))
                        Icon(
                            painter = painterResource(id = R.drawable.carrot_24),
                            contentDescription = context.resources.getText(R.string.vegetarian).toString()
                        )
                    }
                }
            }
        }
    }
}
