package de.ciluvien.mensen.ui.screens

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.pulltorefresh.rememberPullToRefreshState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.vectorResource
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import de.ciluvien.mensen.MensenApp
import de.ciluvien.mensen.R
import de.ciluvien.mensen.ui.Screen
import de.ciluvien.mensen.ui.components.CanteenList
import de.ciluvien.mensen.ui.components.core.BottomNavigationBar
import de.ciluvien.mensen.ui.components.core.CustomSettingsTopAppBar
import de.ciluvien.mensen.ui.components.core.CustomTextIcon
import de.ciluvien.mensen.ui.components.core.NothingFoundRefresh
import de.ciluvien.mensen.ui.viewModelFactory
import de.ciluvien.mensen.ui.viewmodels.CanteenListViewModel
import sh.calvin.reorderable.rememberReorderableLazyListState

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CanteensScreen(navController: NavController) {
    val context = LocalContext.current
    val canteenViewModel = viewModel<CanteenListViewModel>(
        factory = viewModelFactory {
            CanteenListViewModel(MensenApp.appModule)
        }
    )

    LaunchedEffect(key1 = true) {
        canteenViewModel.refreshState()
    }

    val pullToRefreshState = rememberPullToRefreshState()
    if (pullToRefreshState.isRefreshing) {
        LaunchedEffect(key1 = true) {
            canteenViewModel.forceRemoteRefreshSuspend()
            pullToRefreshState.endRefresh()
        }
    }

    val lazyListState = rememberLazyListState()
    val reorderableLazyListState = rememberReorderableLazyListState(lazyListState = lazyListState) { from, to ->
        canteenViewModel.changePosition(from.index, to.index)
    }

    val scrollBehavior = TopAppBarDefaults.pinnedScrollBehavior()
    Scaffold(
        modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
        topBar = {
            CustomSettingsTopAppBar(
                navController = navController,
                title = context.resources.getText(R.string.canteens_title).toString(),
                scrollBehavior = scrollBehavior,
                leftIcon = CustomTextIcon(
                    onClick = { navController.navigate(Screen.CardScreen.route) },
                    icon = ImageVector.vectorResource(id = R.drawable.baseline_payment_24),
                    description = context.resources.getText(R.string.card_title).toString()
                ),
            )
        },
        bottomBar = { BottomNavigationBar(navController) }
    ) {padding ->
        Box(modifier = Modifier.padding(padding)){
            if (canteenViewModel.state.value.canteensAndPositions.isEmpty()) {
                NothingFoundRefresh(text = context.resources.getText(R.string.canteens_no_canteen).toString()) {
                    canteenViewModel.forceRemoteRefresh()
                }
            } else {
                CanteenList(
                    navController = navController,
                    pullToRefreshState = pullToRefreshState,
                    scrollBehavior = scrollBehavior,
                    listState = lazyListState,
                    reorderableLazyListState = reorderableLazyListState,
                    canteenList = canteenViewModel.state.value.canteensAndPositions
                )
            }
        }
    }
}
