package de.ciluvien.mensen.ui.screens

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.outlined.ArrowBack
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.platform.LocalContext
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import de.ciluvien.mensen.MensenApp
import de.ciluvien.mensen.R
import de.ciluvien.mensen.ui.components.MealDetails
import de.ciluvien.mensen.ui.components.core.CustomTextIcon
import de.ciluvien.mensen.ui.components.core.CustomTopAppBar
import de.ciluvien.mensen.ui.viewModelFactory
import de.ciluvien.mensen.ui.viewmodels.MealDetailsViewModel

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun MealDetailsScreen(
    navController: NavController,
    canteenName: String,
    mealId: Int
) {
    val context = LocalContext.current
    val viewModel = viewModel<MealDetailsViewModel>(
        factory = viewModelFactory {
            MealDetailsViewModel(MensenApp.appModule)
        }
    )
    LaunchedEffect(key1 = true) {
        viewModel.getMeal(mealId)
    }

    val scrollBehavior = TopAppBarDefaults.pinnedScrollBehavior()
    Scaffold(
        modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
        topBar = {
            CustomTopAppBar(
                title = canteenName,
                scrollBehavior = scrollBehavior,
                leftIcon = CustomTextIcon(
                    onClick = { navController.popBackStack() },
                    icon = Icons.AutoMirrored.Outlined.ArrowBack,
                    description = context.resources.getText(R.string.meal_back).toString()
                )
            )
        }
    ) {padding ->
        Box(modifier = Modifier.padding(padding)) {
            MealDetails(
                meal = viewModel.meal.value,
                scrollBehavior = scrollBehavior,
                navController = navController
            )
        }
    }
}