package de.ciluvien.mensen.ui.viewmodels

import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import de.ciluvien.mensen.AppModule
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch

class SettingsViewModel(appModule: AppModule) : ViewModel() {
    private val settings = appModule.settings

    val isVegan = settings.isVeganFlow.stateIn(
        viewModelScope,
        SharingStarted.WhileSubscribed(5000L),
        false
    )
     val isVegetarian = settings.isVegetarianFlow.stateIn(
        viewModelScope,
        SharingStarted.WhileSubscribed(5000L),
        false
    )

    fun toggleVegan() {
        viewModelScope.launch {
            if(isVegan.value) {
                settings.disableVegan()
            } else {
                settings.enableVegan()
            }
        }
    }

    fun toggleVegetarian() {
        viewModelScope.launch {
            if(isVegetarian.value) {
                settings.disableVegetarian()
            } else {
                settings.enableVegetarian()
            }
        }
    }

    private fun setVegan() {
        viewModelScope.launch {
            settings.enableVegan()
            settings.disableVegetarian()
        }
    }

    private fun setVegetarian() {
        viewModelScope.launch {
            settings.enableVegetarian()
            settings.disableVegan()
        }
    }

    private fun setEverything() {
        viewModelScope.launch {
            settings.disableVegan()
            settings.disableVegetarian()
        }
    }

    enum class Filter() {
        VEGAN,
        VEGETARIAN,
        EVERYTHING
    }

    private fun getFilter(): Filter {
        return if (isVegan.value) {
            Filter.VEGAN
        } else if (isVegetarian.value) {
            Filter.VEGETARIAN
        } else {
            Filter.EVERYTHING
        }
    }

    fun setFilter(ordinal: Int) {
        when(ordinal){
            Filter.EVERYTHING.ordinal -> setEverything()
            Filter.VEGETARIAN.ordinal -> setVegetarian()
            Filter.VEGAN.ordinal -> setVegan()
        }
    }
}